
-- ----------------------------------------------------------
--   Useful information goes here at the top, like Copyright 
--   notices, authors, creation dates, etc..  All of this is 
--   ingnored by the compiler.
-- ----------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity push_button is
  port 
   (
    clk            : in  std_logic; --50MHz
    pb_in          : in  std_logic; --low-active
    --
    led            : out std_logic
   );
end entity push_button;

architecture Behavioral of push_button is

   constant SIM          : boolean := true;
   constant MS_LOAD      : unsigned(15 downto 0) := X"C350";
   constant MS_LOAD_SIM  : unsigned(15 downto 0) := X"0002";
   constant DEBOUNCE     : unsigned(7 downto 0)  := X"04";

   signal counter        : unsigned(27 downto 0) := X"0000000";
   signal ms_count       : unsigned(15 downto 0) := X"0001";
   signal pb_count       : unsigned(7 downto 0)  := X"00";
   signal pb_d1          : std_logic := '1';
   signal ms_tick        : std_logic := '0';
   signal pb_done        : std_logic := '0';
   signal pb_active      : std_logic := '0';
   signal pb_active_d1   : std_logic := '0';
   signal pb_toggle      : std_logic := '0';
   
begin
   
   led_counter : process(clk) 
   begin
      if rising_edge(clk) then
        counter <= counter + 1;
      end if;
   end process;
   
   led <= pb_toggle AND counter(27);
     
   pb_process : process(clk) 
   begin
      if rising_edge(clk) then
         if (ms_count = X"0001") then
            if (SIM = true) then
               ms_count <= MS_LOAD_SIM;
            else
               ms_count <= MS_LOAD;
            end if;
            --
            ms_tick  <= '1';
         else
            ms_count <= ms_count - 1;
            ms_tick  <= '0';
         end if;
         --
         pb_d1 <= pb_in;  --synchronize
         --
         if (pb_d1 = '0') then
            pb_count <= DEBOUNCE;
         elsif (ms_tick = '1') then
            if (pb_count /= X"00") then
               pb_count <= pb_count - 1;
            end if;
            --
            if (pb_count = X"01") then
               pb_done <= '1';
            else 
               pb_done <= '0';
            end if;
         end if;
         --
         if (pb_d1 = '0') then
            pb_active <= '1';
         elsif (pb_done = '1') then
            pb_active <= '0';
         end if;
      end if;
   end process;
   
   -- falling edge detection
   pb_toggler : process(clk) 
   begin
      if rising_edge(clk) then
         pb_active_d1 <= pb_active;
         --
         if ( pb_active = '1' AND pb_active_d1 = '0' ) then
            pb_toggle <= not pb_toggle;
         end if;
      end if;
   end process;
   
   
end architecture Behavioral;



