
-- ----------------------------------------------------------
--   Useful information goes here at the top, like Copyright 
--   notices, authors, creation dates, etc..  All of this is 
--   ingnored by the compiler.
-- ----------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity push_button_2 is
  port 
   (
    clk            : in  std_logic; --50MHz
    pb_in          : in  std_logic; --low-active
    --
    led            : out std_logic
   );
end entity push_button_2;

architecture Behavioral of push_button_2 is

   -- ~1 million
   constant DEBOUNCE     : unsigned(23 downto 0) := X"100000";

   signal counter_led    : unsigned(27 downto 0) := X"0000000";
   signal pb_count       : unsigned(23 downto 0) := X"000000";
   signal pb_d1          : std_logic := '1';
   signal pb_active      : std_logic := '0';
   signal pb_active_d1   : std_logic := '0';
   signal pb_toggle      : std_logic := '0';
   
begin
   pb_process : process(clk) 
   begin
      if rising_edge(clk) then
         pb_d1 <= pb_in;  --synchronize
         --
         if (pb_d1 = '0') then
            pb_count <= DEBOUNCE;
         elsif (pb_count /= X"000000") then
            pb_count <= pb_count - 1;
         end if;
         --
         if (pb_d1 = '0') then
            pb_active <= '1';
         elsif (pb_count = X"000001") then
            pb_active <= '0';
         end if;
         -- rising edge detection and toggle
         pb_active_d1 <= pb_active;
         if ( pb_active = '1' AND pb_active_d1 = '0' ) then
            pb_toggle <= not pb_toggle;
         end if;
      end if;
   end process;

   led_counter : process(clk) 
   begin
      if rising_edge(clk) then
        counter_led <= counter_led + 1;
      end if;
   end process;
   
   led <= pb_toggle AND counter_led(27);
   
end architecture Behavioral;



