
-- ----------------------------------------------------------
--   Useful information goes here at the top.
-- ----------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity debounce is
  port 
   (
     clk            : in  std_logic;
     pb_in          : in  std_logic;
     ms_tick        : in  std_logic;
     pb_debnced     : out std_logic
   );
end entity debounce;

architecture Behavioral of debounce is

   -- ~1 million
   constant DEBOUNCE_TIME : unsigned(11 downto 0) := X"3E8";

   signal pb_d1          : std_logic;
   signal pb_active      : std_logic;
   signal pb_active_d1   : std_logic;
   signal pb_count       : unsigned(11 downto 0) := X"000";
   
begin

   pb_process : process(clk) 
   begin
      if rising_edge(clk) then
         pb_d1 <= pb_in;  --synchronize
         --
         if (pb_d1 = '0') then
            pb_count <= DEBOUNCE_TIME;
         elsif (ms_tick = '1') then
            if (pb_count /= X"000") then
               pb_count <= pb_count - 1;
            end if;
            --
         end if;
         --
         if (pb_d1 = '0') then
            pb_active <= '1';
         elsif (pb_count = X"001") then
            pb_active <= '0';
         end if;
         -- rising edge detection and toggle
         pb_active_d1 <= pb_active;
         if ( pb_active = '1' AND pb_active_d1 = '0' ) then
            pb_debnced <= '1';
         else 
            pb_debnced <= '0';
         end if;
      end if;
   end process;
   
end architecture Behavioral;



