--  Push Button Testbench
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity target_game_tb is
end entity target_game_tb;

architecture Testbench of target_game_tb is

   component target_game_sim is
     generic
      (  simulation : boolean
      );
     port 
      (
       clk            : in  std_logic; --50MHz
       pb_in          : in  std_logic; --low-active
       --
       led            : out std_logic
      );
   end component;

   signal clk            : std_logic := '0';
   signal pb_in          : std_logic := '1';
   signal tb_counter     : unsigned(15 downto 0) := X"0000";
   signal led            : std_logic;
   
begin

   ------  generate clock ------------
   
   clk  <= '0' after 10 ns when clk = '1' else   -- 50MHz = 20ns
           '1' after 10 ns;
           
   ------  Push-Button generation
   
   p_button : process(clk) 
   begin
      if rising_edge(clk) then
         tb_counter <= tb_counter + 1;
         --
         case(tb_counter) is
            when X"000D" => pb_in <= '0';
            when X"000F" => pb_in <= '1';
            when X"0028" => pb_in <= '0';
            when X"002B" => pb_in <= '1';
            when others => null;
         end case;
      end if;
   end process;

   ----- design instantiation
   
   target_game_sim_inst : target_game_sim
     generic map 
      ( simulation => true
      )
     port map
      (
        clk            => clk,   --in  std_logic; --50MHz
        pb_in          => pb_in, --in  std_logic; --low-active
        --
        led            => led    --out std_logic
      );
           
end architecture Testbench;


