--  Push Button Testbench
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity uart_both_tb is
end entity uart_both_tb;

architecture Testbench of uart_both_tb is

  component uart_tx is
  port 
   (
     clk           : in   std_logic;
     uart_tx_dat   : in   std_logic_vector(6 downto 0);
     uart_tx_go    : in   std_logic;
     tx_sig        : out  std_logic;
     trdy          : out  std_logic
   );
  end component;

  component uart_rx is
  port (
        clk           : in   std_logic;
        rx_sig        : in   std_logic;
        uart_rx_dat   : out  std_logic_vector(7 downto 0);
        fifo_read     : in   std_logic;
        rrdy          : out  std_logic := '0'
        );
  end component;

   type tx_sm_type is (
                       tx_look_1,
                       tx_go_1,
                       tx_wait_1,
                       tx_look_2,
                       tx_go_2,
                       tx_wait_2,
                       tx_look_3,
                       tx_go_3,
                       tx_wait_end
                      );
                       
   signal tx_sm : tx_sm_type;

   type rx_sm_type is (
                        rx_wait_1,
                        rx_fifo_rd_1,
                        rx_pause_1,
                        rx_wait_2,
                        rx_fifo_rd_2,
                        rx_pause_2,
                        rx_wait_3,
                        rx_fifo_rd_3,
                        rx_wait_end
                       );
                       
   signal rx_sm : rx_sm_type;

   signal clk             : std_logic := '0';
   signal uart_tx_dat     : std_logic_vector(6 downto 0);
   signal uart_tx_go      : std_logic := '0';
   signal trdy            : std_logic;
   signal uart_sig        : std_logic;
   signal rrdy            : std_logic;
   signal uart_rx_dat     : std_logic_vector(7 downto 0);
   signal fifo_read       : std_logic;
   
begin

   ---------------------------------------------------------------------
   ------  generate clocks
   ---------------------------------------------------------------------
   
   clk  <= '0' after 10 ns when clk = '1' else   -- 50MHz = 20ns
           '1' after 10 ns;

   ---------------------------------------------------------------------
   ------  Tx Data Generation
   ---------------------------------------------------------------------

   tx_dat_gen_state_machine : process(clk) 
   begin
      if rising_edge(clk) then
         case(tx_sm) is
            when tx_look_1   => if (trdy = '1') then tx_sm <= tx_go_1; end if;
            when tx_go_1     =>                      tx_sm <= tx_wait_1;
            when tx_wait_1   =>                      tx_sm <= tx_look_2;
            when tx_look_2   => if (trdy = '1') then tx_sm <= tx_go_2; end if;
            when tx_go_2     =>                      tx_sm <= tx_wait_2;
            when tx_wait_2   =>                      tx_sm <= tx_look_3;
            when tx_look_3   => if (trdy = '1') then tx_sm <= tx_go_3; end if;
            when tx_go_3     =>                      tx_sm <= tx_wait_end;
            when tx_wait_end => null;
            when others => null;
         end case;
      end if;
   end process;

   tx_dat_gen_process : process(tx_sm) 
   begin
      case(tx_sm) is
         when tx_go_1 => uart_tx_go  <= '1'; 
                         uart_tx_dat <= "1011001";
         when tx_go_2 => uart_tx_go  <= '1'; 
                         uart_tx_dat <= "0100110";   
         when tx_go_3 => uart_tx_go  <= '1'; 
                         uart_tx_dat <= "1100110";   
         when others =>  uart_tx_go  <= '0'; 
                         uart_tx_dat <= "0000000";
      end case;
   end process;

   ---------------------------------------------------------------------
   ------  Rx FIFO Reads
   ---------------------------------------------------------------------
   
   rx_fifo_rd_state_machine : process(clk) 
   begin
      if rising_edge(clk) then
         case(rx_sm) is
            when rx_wait_1    => if (rrdy = '1') then rx_sm <= rx_fifo_rd_1; end if;
            when rx_fifo_rd_1 =>                      rx_sm <= rx_pause_1;
            when rx_pause_1   =>                      rx_sm <= rx_wait_2;
            when rx_wait_2    => if (rrdy = '1') then rx_sm <= rx_fifo_rd_2; end if; 
            when rx_fifo_rd_2 =>                      rx_sm <= rx_pause_2; 
            when rx_pause_2   =>                      rx_sm <= rx_wait_2;
            when rx_wait_3    => if (rrdy = '1') then rx_sm <= rx_fifo_rd_3; end if; 
            when rx_fifo_rd_3 =>                      rx_sm <= rx_wait_end; 
            when rx_wait_end  => null;
            when others => null;
         end case;
      end if;
   end process;

   rx_fifo_read_process : process(rx_sm) 
   begin
      case(rx_sm) is
         when rx_fifo_rd_1 => fifo_read  <= '1'; 
         when rx_fifo_rd_2 => fifo_read  <= '1'; 
         when rx_fifo_rd_3 => fifo_read  <= '1'; 
         when others => fifo_read  <= '0';
      end case;
   end process;

   ---------------------------------------------------------------------
   ------  module instantiations
   ---------------------------------------------------------------------
   
  uart_tx_i : uart_tx
  port map
   (
     clk           => clk,         --in   std_logic;
     uart_tx_dat   => uart_tx_dat, --in   std_logic_vector(7 downto 0);
     uart_tx_go    => uart_tx_go,  --in   std_logic;
     tx_sig        => uart_sig,    --out  std_logic;
     trdy          => trdy         --out  std_logic
   );

  uart_rx_i : uart_rx
  port map
   (
     clk           => clk,
     rx_sig        => uart_sig,
     uart_rx_dat   => uart_rx_dat,
     fifo_read     => fifo_read,
     rrdy          => rrdy
   );
   
end architecture Testbench;


