--  Push Button Testbench
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity uart_tx_tb is
end entity uart_tx_tb;

architecture Testbench of uart_tx_tb is

  component uart_tx is
  port 
   (
     clk           : in   std_logic;
     uart_tx_dat   : in   std_logic_vector(6 downto 0);
     uart_tx_go    : in   std_logic;
     tx_sig        : out  std_logic;
     trdy          : out  std_logic
   );
  end component;
  
   signal clk             : std_logic := '0';
   signal run_counter     : std_logic_vector(7 downto 0) := X"00";
   signal uart_tx_go      : std_logic := '0';
   signal uart_tx_dat     : std_logic_vector(6 downto 0) := "1011001";
   signal tx_sig          : std_logic;
   signal trdy            : std_logic;
   
begin

   ---------------------------------------------------------------------
   ------  generate clocks
   ---------------------------------------------------------------------
   
   clk  <= '0' after 10 ns when clk = '1' else   -- 50MHz = 20ns
           '1' after 10 ns;

   run_counter_process : process(clk) 
   begin
      if rising_edge(clk) then
         run_counter <= run_counter + 1;
         --
         case(run_counter) is
            when X"01" => uart_tx_go <= '1';
            when X"02" => uart_tx_go <= '0';
            when others => uart_tx_go <= '0';
         end case;
      end if;
   end process;

  uart_tx_i : uart_tx
  port map
   (
     clk           => clk,         --in   std_logic;
     uart_tx_dat   => uart_tx_dat, --in   std_logic_vector(7 downto 0);
     uart_tx_go    => uart_tx_go,  --in   std_logic;
     tx_sig        => tx_sig,      --out  std_logic;
     trdy          => trdy         --out  std_logic
   );
           
end architecture Testbench;


