--  Push Button Testbench
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity shell_tb is
end entity shell_tb;

architecture Testbench of shell_tb is

   -- reduced for simulation
   constant BAUD_9600 : std_logic_vector(15 downto 0) := X"0010";
   
  component shell is
  generic (BAUD_9600 : std_logic_vector(15 downto 0));
  port (
        clk           : in   std_logic;
        address       : in   std_logic_vector(11 downto 0);
        wr_en         : in   std_logic;
        data_wr       : in   std_logic_vector(7 downto 0);
        rd_en         : in   std_logic;
        data_rd       : out  std_logic_vector(7 downto 0);
        --
        uart_tx_sig   : out  std_logic;
        uart_rx_sig   : in   std_logic;
        leds          : out  std_logic_vector(3 downto 0)
        );
   end component;

   signal run_counter     : std_logic_vector(11 downto 0) := X"000";
   --UUT inputs (generated by the testbench)
   signal clk             : std_logic := '0';
   signal address       : std_logic_vector(11 downto 0) := X"000";
   signal wr_en         : std_logic;
   signal data_wr       : std_logic_vector(7 downto 0) := X"00";
   signal rd_en         : std_logic := '0';
   signal rrdy          : std_logic;
   signal uart_rx_sig   : std_logic;
   -- UUT outputs (observed by the testbench)
   signal data_rd       : std_logic_vector(7 downto 0);
   signal uart_tx_sig   : std_logic;
   signal leds          : std_logic_vector(3 downto 0) := X"0";

   
begin

   clk  <= '0' after 10 ns when clk = '1' else   -- 50MHz = 20ns
           '1' after 10 ns;

   run_sequence : process 
      variable i : integer := 0;
   begin
      for i in 1 to 3 loop wait until rising_edge(clk); end loop;
      address <= X"006"; --LEDs
      data_wr <= X"05";
      wr_en   <= '1';
      wait until rising_edge(clk);
      wr_en   <= '0';
      --
      for i in 1 to 3 loop wait until rising_edge(clk); end loop;
      address <= X"800"; --memory
      data_wr <= X"AA";
      wr_en   <= '1';
      wait until rising_edge(clk);
      wr_en   <= '0';
      -- write to memory
      for i in 1 to 7 loop wait until rising_edge(clk); end loop;
      address <= X"921"; --memory
      data_wr <= X"BB";
      wr_en   <= '1';
      wait until rising_edge(clk);
      wr_en   <= '0';
      -- read from memory
      for i in 1 to 7 loop wait until rising_edge(clk); end loop;
      address <= X"800"; --memory
      wait until rising_edge(clk);
      rd_en   <= '1';
      for i in 1 to 2 loop wait until rising_edge(clk); end loop;
      rd_en   <= '0';
      for i in 1 to 2 loop wait until rising_edge(clk); end loop;
      address <= X"921"; --memory
      wait until rising_edge(clk);
      rd_en   <= '1';
      for i in 1 to 2 loop wait until rising_edge(clk); end loop;
      rd_en   <= '0';
      --
      for i in 1 to 8 loop wait until rising_edge(clk); end loop;
      address <= X"001"; --UART Tx
      data_wr <= X"09";
      wr_en   <= '1';
      for i in 1 to 4 loop wait until rising_edge(clk); end loop;
      wr_en   <= '0';
      --
      loop
         for i in 1 to 30 loop wait until rising_edge(clk); end loop;
         address <= X"003"; --rrdy
         rd_en   <= '1';
         for i in 1 to 2 loop wait until rising_edge(clk); end loop;
         if (data_rd(0) = '1') then
            wait until rising_edge(clk);
            address <= X"004"; --uart rx FIFO read
            wr_en   <= '1';
            for i in 1 to 4 loop wait until rising_edge(clk); end loop;
            wr_en   <= '0';
            wait until rising_edge(clk);
            address <= X"005"; --uart_rx_dat
            rd_en   <= '1';
            for i in 1 to 2 loop wait until rising_edge(clk); end loop;
            rd_en   <= '0';
         end if;
         wait until rising_edge(clk);
         rd_en   <= '0';
      end loop;
   end process;

  shell_inst : shell
  generic map (BAUD_9600 => BAUD_9600)
  port map 
   (
     clk           => clk,        
     address       => address,    
     wr_en         => wr_en,      
     data_wr       => data_wr,    
     rd_en         => rd_en,      
     data_rd       => data_rd,    
     --
     uart_tx_sig   => uart_tx_sig,
     uart_rx_sig   => uart_tx_sig,
     leds          => leds        
   );
           
end architecture Testbench;


