
-- ------------------------------------------------------
--   (Useful information.)
-- ------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity bus_intfc is 
  port (
         clk           : in   std_logic;
         address       : in   std_logic_vector(11 downto 0);
         wr_en         : in   std_logic;
         data_wr       : in   std_logic_vector(7 downto 0);
         rd_en         : in   std_logic;
         data_rd       : out  std_logic_vector(7 downto 0);
         --            
         trdy          : in   std_logic;
         uart_tx_go    : out  std_logic;
         uart_tx_dat   : out  std_logic_vector(7 downto 0);
         --            
         rrdy          : in   std_logic;
         fifo_read     : out  std_logic;
         uart_rx_dat   : in   std_logic_vector(7 downto 0);
         --            
         leds          : out  std_logic_vector(3 downto 0);
         --            
         memory_adr    : out  std_logic_vector(10 downto 0);
         memory_wr     : out  std_logic;
         memory_wr_dat : out  std_logic_vector(7 downto 0);
         memory_rd_dat : in   std_logic_vector(7 downto 0)
       );
end entity bus_intfc;

architecture Behavioral of bus_intfc is

   signal data_wr_d1            : std_logic_vector(7 downto 0);
   signal uart_tx_go_decode     : std_logic;
   signal fifo_read_decode      : std_logic;
   signal uart_tx_go_decode_d1  : std_logic;
   signal fifo_read_decode_d1   : std_logic;

begin

   writes : process(clk)
   begin
     if rising_edge(clk) then
        data_wr_d1 <= data_wr;
        --
        uart_tx_go_decode <= '0';
        fifo_read_decode  <= '0';
        memory_wr         <= '0';
        
        if (wr_en = '1') then
           case (address) is
             when X"001"         => uart_tx_go_decode <= '1';
             when X"004"         => fifo_read_decode  <= '1';
             when X"006"         => leds <= data_wr(3 downto 0);
             when others         =>   --memory
                                    if (address(11) = '1') then memory_wr <= '1';
                                    else null;
                                    end if;
           end case;
        end if;
     end if;
   end process;

   rising_edge_proc : process(clk)
   begin
     if rising_edge(clk) then
        uart_tx_go_decode_d1 <= uart_tx_go_decode;
        fifo_read_decode_d1  <= fifo_read_decode;
        --
        uart_tx_go <= uart_tx_go_decode AND NOT uart_tx_go_decode_d1;
        fifo_read  <= fifo_read_decode AND NOT fifo_read_decode_d1;
     end if;
   end process;

   reads : process(clk)
   begin
     if rising_edge(clk) then
        data_rd <= X"00";
        --
        if (rd_en = '1') then
           case (address) is
             when X"000"         => data_rd <= "0000000" & trdy;
             when X"003"         => data_rd <= "0000000" & rrdy;
             when X"005"         => data_rd <= uart_rx_dat;
             when others         =>   --memory
                                    if (address(11) = '1') then data_rd <= memory_rd_dat;
                                    else null;
                                    end if;
           end case;
        end if;
     end if;
   end process;
   
   uart_tx_dat   <= data_wr_d1;
   memory_adr    <= address(10 downto 0);
   memory_wr_dat <= data_wr_d1;
   
end architecture Behavioral;


