
-- ------------------------------------------------------
--   (Useful information.)
-- ------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity fifo is
  port (
         clk           : in   std_logic;
         dat_in        : in   std_logic_vector(7 downto 0);
         f_write       : in   std_logic;
         dat_out       : out  std_logic_vector(7 downto 0);
         f_read        : in   std_logic;
         empty         : out  std_logic
       );
end entity fifo;

architecture Behavioral of fifo is

   type memory_type is array(0 to 7) of std_logic_vector(7 downto 0);
   signal memory : memory_type;
   
   signal write_adr         : unsigned(2 downto 0) := "000";
   signal read_adr          : unsigned(2 downto 0) := "000";
   signal write_adr_fifo    : integer;
   signal read_adr_fifo     : integer;
   
begin

   fifo_addresses : process(clk)
   begin
      if rising_edge(clk) then
         if (f_write = '1') then
            write_adr <= write_adr + 1;
         end if;
         --
         if (f_read = '1') then
            read_adr <= read_adr + 1;
         end if;
      end if;
   end process;

   write_adr_fifo <= to_integer(write_adr + 1);
   read_adr_fifo  <= to_integer(read_adr);
   
   memory_process : process(clk)
   begin
      if rising_edge(clk) then
         if (f_write = '1') then
            memory(write_adr_fifo) <= dat_in;
         end if;
         --
         dat_out <= memory(read_adr_fifo);
      end if;
   end process;
   
   empty <= '1' when (write_adr = read_adr) else '0';
   
end architecture Behavioral;


