
-- ------------------------------------------------------
--   (Useful information.)
-- ------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity uart_tx is
  generic (BAUD_9600 : std_logic_vector(15 downto 0));
  port (
        clk           : in   std_logic;
        uart_tx_dat   : in   std_logic_vector(6 downto 0);
        uart_tx_go    : in   std_logic;
        tx_sig        : out  std_logic;
        trdy          : out  std_logic := '0'
        );
end entity uart_tx;

architecture Behavioral of uart_tx is

   component baud is
     port (
            clk           : in   std_logic;
            baud_config   : in   std_logic_vector(15 downto 0);
            baud_en       : out  std_logic
          );
   end component;
   
   type tx_sm_type is (
                        tx_idle,
                        tx_go,
                        tx_start,
                        tx_actv,
                        tx_stop
                      );
   signal tx_sm : tx_sm_type;
   
   signal baud_en             : std_logic;
   signal tx_bit_cnt          : unsigned(3 downto 0) := X"0";
   signal tx_dat              : std_logic_vector(6 downto 0) := "0000000";
   signal tx_sr               : std_logic_vector(6 downto 0) := "0000000";
   
begin

   baud_inst : baud
     port map
       (
         clk          => clk,       --in  std_logic;
         baud_config  => BAUD_9600, --in  std_logic_vector(15 downto 0);
         baud_en      => baud_en    --out std_logic
       );
   
   uart_tx_bit_cnt : process(clk)
   begin
      if rising_edge(clk) then
         if (baud_en = '1') then
            if (tx_sm = tx_start) then
               tx_bit_cnt <= X"0";
            elsif (tx_bit_cnt /= X"7") then
               tx_bit_cnt <= tx_bit_cnt + 1;
            end if;
         end if;
      end if;
   end process;
   
   uart_tx_sm : process(clk)
   begin
     if rising_edge(clk) then
       case (tx_sm) is
         when tx_idle =>  if (uart_tx_go = '1') then      tx_sm <= tx_go;      end if;
         when tx_go =>    if (baud_en = '1') then         tx_sm <= tx_start;   end if;
         when tx_start => if (baud_en = '1') then         tx_sm <= tx_actv;    end if;
         when tx_actv =>  if (baud_en = '1') then
                             if (tx_bit_cnt = X"6") then  tx_sm <= tx_stop;    end if;
                          end if;
         when tx_stop =>  if (baud_en = '1') then         tx_sm <= tx_idle;    end if;
         when others => null;
       end case;
     end if;
   end process;
   
   uart_tx_proc : process(clk)
   begin
      if rising_edge(clk) then
         if (uart_tx_go = '1') then
            tx_dat <= uart_tx_dat;
         end if;
         --
         if (tx_sm = tx_idle) then
            trdy <= '1';
         else
            trdy <= '0';
         end if;
         --
         if (baud_en = '1') then
            --
            if (tx_sm = tx_go) then
               tx_sr <= tx_dat;
            elsif (tx_sm = tx_actv) then
               tx_sr <= '0' & tx_sr(6 downto 1);
            end if;
         end if;
      end if;
   end process;

   tx_sig <= '1'      when (tx_sm = tx_idle)  else
             '1'      when (tx_sm = tx_go)    else
             '0'      when (tx_sm = tx_start) else
             tx_sr(0) when (tx_sm = tx_actv)  else
             '1'      when (tx_sm = tx_stop)  else
             '1';
   
end architecture Behavioral;


